/*
   MP3 Index Creator v. 0.1 - Copyright (C) 1998 by Jeremy S. Johnstone

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
*/

#include <unistd.h>
#include <stdio.h>
#include <dirent.h>
#include <string.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <statbuf.h>

#define INDEXNAME "index"

void getnames(char *dir, int depth, char listarray[][50]);
void get_description(int depth, char *dirname, char *filesize, char listarray[][50]);
int current_row = 0;
double total_size = 0;

int main(int argc, char *argv)
{
   FILE *outfile;
   char listarray[300][50];
   int i;
   char title[20];
   char *workingdir = (char *) getenv("PWD");

   printf("What is the name of this collection? ");
   scanf("%s", title);
   getchar();
   printf("Please input descriptions. (Max of 30 characters each)\n");
   getnames(workingdir, 0, listarray);
   chdir(workingdir);
   outfile = fopen(INDEXNAME, "w");
   fprintf(outfile, "MP3 Index Creator v0.1 - Copyright 1998 by Jeremy Johnstone\n");
   fprintf(outfile, "-------------------------------------------------------------\n\n\n");
   fprintf(outfile, "Collection name - %s\tTotal size = %3.2lf Mb\tNumber of files/dirs = %d\n\n", title, total_size, current_row / 3);
   fprintf(outfile, "Filename\t\t\tSize\t  Description\n");
   fprintf(outfile, "--------------------------------------------------------------------------\n");

   for(i=0;i<current_row;i=i+3)
      fprintf(outfile, "%-30.30s\t%-7.7s\t  %s", listarray[i],listarray[i+1],listarray[i+2]);

   return 0;
}

void getnames(char *dir, int depth, char listarray[][50])
{
   DIR *dp;
   struct dirent *entry;
   struct stat statbuf;
   double blocks;
   char filesize[10];
   char dirname[20];

   if ((dp = opendir(dir)) == NULL) {
      fprintf(stderr, "Cannot open directory: %s\n", dir);
      return;
   }
   
   chdir(dir);
   while((entry = readdir(dp)) != NULL) {
      stat(entry->d_name, &statbuf);
      blocks = ((statbuf).st_blocks) / 2.0 / 1024;
      total_size += blocks;
      sprintf(filesize, "%2.2lf Mb", blocks);
      if(S_ISDIR(statbuf.st_mode)) {
         /* Found a directory, but ignore . and .. */
         if(strcmp(".",entry->d_name) == 0 || strcmp("..", entry->d_name) == 0)      
            continue;
         sprintf(dirname, "\n%s/", entry->d_name);
         get_description(depth, dirname, "0", listarray);
         /* recurse at a new indent level */
         getnames(entry->d_name,depth+4,listarray);
      }
      else {
         get_description(depth, entry->d_name, filesize, listarray);
      }
   }
   chdir("..");
   closedir(dp);
}

void get_description(int depth, char *dirname, char *filesize, char listarray[][50])
{
   sprintf(listarray[current_row], "%*s%s", depth, "", dirname);
   current_row++;
   if (filesize != "0")
      strcpy(listarray[current_row], filesize);
   else
      strcpy(listarray[current_row],"");
   printf("Please enter a description for - %s:\n", listarray[current_row-1]);
   current_row++;
   fgets(listarray[current_row], 38, stdin); 
   current_row++;
}
